@@ -2,6 +2,7 @@ from ibidas.utils import util
 from ibidas.utils.config import config
 import os.path
 import os
+from logging import warning
 _delay_import_(globals(),"ibidas","*")
 
 class Pre(object):
@@ -136,15 +137,16 @@ predefined_sources.register(yeastract,category="yeast")
 
 def omim_genemap():
     """The omim genemap data"""
-    rtype = """[omim:*]<(chr_map_entry_nr=bytes, month=bytes, day=bytes, year=bytes, location=bytes, gene_names=bytes, 
+    rtype = """[omim:*]<(chr_map_entry_nr=bytes, month=bytes, day=bytes, year=bytes, location=bytes, gene_alias=bytes, 
                        gene_status=bytes[1], title=bytes, f8=bytes, mim=bytes, method=bytes, comments=bytes, f12=bytes, 
                        disease1=bytes, disease2=bytes, disease3=bytes, mouse=bytes, reference=bytes)"""
-    
-    res = Read(Fetch("ftp://ftp.ncbi.nih.gov/repository/OMIM/genemap"),dtype=rtype)
+
+    warning('OMIM FTP is not up to date. See ftp://ftp.ncbi.nih.gov/repository/OMIM/ARCHIVE/README.txt')
+    res = Read(Fetch("ftp://ftp.ncbi.nih.gov/repository/OMIM/ARCHIVE/genemap"),dtype=rtype)
     
     res = res.Get(HArray(_.disease1, _.disease2, _.disease3)[_ != " "]/"disease", "~")
     splitfunc = lambda x: x.split(", ")
-    res = res.To(_.gene_names,  Do=_.Each(splitfunc, dtype="[symbols:~]<string").Elem()[_ != ""])
+    res = res.To(_.gene_alias,  Do=_.Each(splitfunc, dtype="[symbols:~]<string").Elem()[_ != ""])
     res = res.To(_.method,      Do=_.Each(splitfunc, dtype="[methods:~]<string").Elem()[_ != ""])
     res = res.To(_.disease,     Do=_.Sum().Each(omim_disease_parse,"[diseases:~]<string").Elem()[_ != ""])
     res = res.To(_.day, _.month, _.year, Do=_.Cast("int?"))
@@ -154,8 +156,7 @@ def omim_genemap():
 predefined_sources.register(omim_genemap, category="human")
 
 def omim_disease_parse(x):
-    x = x.replace('{','').replace('}','')
-    elems =  x.split('; ')
+    elems = x.replace('{','').replace('}','').split('; ')
     return [elem.split(', ')[0] for elem in elems]
 
 
@@ -271,7 +272,7 @@ def go_annotations(dburl=config.get("databases.go_url",None), genus="Saccharomyc
     return g.Copy()
 predefined_sources.register(go_annotations,name="annotations",category="go")
 
-def go_info(dburl=config.get("databases.go_url",None), genus="Saccharomyces", species="cerevisiae"):
+def go_info(dburl=config.get("databases.go_url",None), genus="Saccharomyces", species="cerevisiae", include_ancestors=False):
     """Accesses GO term info in a MySQL database.
 
        Database data can be obtained from the geneontology website.
@@ -279,6 +280,7 @@ def go_info(dburl=config.get("databases.go_url",None), genus="Saccharomyces", sp
        example url: "mysql://username:password@hostname:port/go
     """
     go = open_go(dburl)
+
     #select annotations from a certain species, and their ancestor annotations
     g = go.species
     g = g |Match(_.id,                  _.species_id)|      go.gene_product
@@ -293,22 +295,47 @@ def go_info(dburl=config.get("databases.go_url",None), genus="Saccharomyces", sp
 
     #get relevant fields
     g = g.ReplaceMissing()
-    g = g.Get(_.child.acc/"go_id", _.parent.acc/"parent_id", _.child.term_type/"go_type", _.child.name/"annotation",
-              _.distance, _.rel.name/"relationship", _.gene_product.id/"gene_id").Copy()
+
+    if include_ancestors:
+        g = g.Get(_.child.acc/"go_id", _.parent.acc/"parent_id", _.gene_product.id/"gene_id").Copy()
+    else:
+        g = g.Get(_.child.acc/"go_id", _.parent.acc/"parent_id", _.child.term_type/"go_type", _.child.name/"annotation",
+                _.distance, _.rel.name/"relationship", _.gene_product.id/"gene_id").Copy()
 
     #step A: calculate number of genes associated with go terms
     ngenes = g.GroupBy(_.parent_id).Get(_.parent_id/"go_id",_.gene_id.Unique().Count()/"ngenes")
 
-    #step B1: group per term
-    goinfo = g.GroupBy(_.go_id, flat=(_.go_type, _.annotation))
-    
-    #step B2: for ech term, determine type, max depth to root, ancestors (non-unique) and relationship to ancestors
-    goinfo = goinfo.Get(_.go_id, _.go_type, _.annotation,  
-                        _[_.parent_id == "all"].distance.Max()/"depth",
-                        _.parent_id / "ancestor", _.relationship, _.distance)
 
-    #step B3: select for each ancestor the minimum path length
-    goinfo = goinfo.GroupBy(_.ancestor)[..., _.distance.Argmin()].Copy()
+    if include_ancestors:
+        g2 = go.graph_path
+        g2 = g2 |Match(_.term2_id,             _.id)| go.term//"child"
+        g2 = g2 |Match(_.term1_id,             _.id)| go.term//"parent"
+        g2 = g2 |Match(_.relationship_type_id, _.id)| go.term//"rel"
+        g2 = g2.Get(_.child.acc/"go_id", _.parent.acc/"parent_id", _.child.term_type/"go_type", _.child.name/"annotation",
+                _.distance, _.rel.name/"relationship").Copy()
+      
+        #step B1: group per term
+        g2 = g2[_.go_id |In| ngenes.go_id]
+        goinfo = g2.GroupBy(_.go_id, flat=(_.go_type, _.annotation))
+        
+        #step B2: for ech term, determine type, max depth to root, ancestors (non-unique) and relationship to ancestors
+        goinfo = goinfo.Get(_.go_id, _.go_type, _.annotation,  
+                            _[_.parent_id == "all"].distance.Max()/"depth",
+                            _.parent_id / "ancestor", _.relationship, _.distance)
+
+        #step B3: select for each ancestor the minimum path length
+        goinfo = goinfo.GroupBy(_.ancestor)[..., _.distance.Argmin()].Copy()
+    else:
+        #step B1: group per term
+        goinfo = g.GroupBy(_.go_id, flat=(_.go_type, _.annotation))
+        
+        #step B2: for ech term, determine type, max depth to root, ancestors (non-unique) and relationship to ancestors
+        goinfo = goinfo.Get(_.go_id, _.go_type, _.annotation,  
+                            _[_.parent_id == "all"].distance.Max()/"depth",
+                            _.parent_id / "ancestor", _.relationship, _.distance)
+
+        #step B3: select for each ancestor the minimum path length
+        goinfo = goinfo.GroupBy(_.ancestor)[..., _.distance.Argmin()].Copy()
 
     #step C: combine, copy, return
     return (goinfo |Match| ngenes).Copy()
@@ -340,7 +367,17 @@ def get_kegg_pathway(pathway):
     elements = Rep(serv.get_elements_by_pathway(pathway))%"nodes"
     pwdata = Combine(elements,relations)
     
-    names = pwdata.names.Flat().Unique()()
+    names = pwdata.names.Flat().Unique()
+    name_info = get_kegg_btit(names)
+    res = (pwdata |Match(_.names, _.id, jointype="left")| name_info).Without(_.id)
+    #info = res.Get((_.names, _.info)).Each(extract_info,dtype="bytes")/"info"
+    #res = res.Without(_.info).Get("*", info)
+    return res.Copy()
+predefined_sources.register(get_kegg_pathway,name="pathway", category="kegg")
+
+def get_kegg_btit(names):
+    serv = kegg()
+    names = names()
     name_info = []
     while len(names) > 0:
         xnames = names[:100]
@@ -349,13 +386,12 @@ def get_kegg_pathway(pathway):
         for elem in res.split('\n'):
             if(elem):
                 pos = elem.index(' ')
-                name_info.append((elem[:pos], elem[(pos+1):]))
-    res = (pwdata |Match(jointype="left")| Rep(name_info)/("names","info")).Without(_.R.names).Copy()
-    #info = res.Get((_.names, _.info)).Each(extract_info,dtype="bytes")/"info"
-    #res = res.Without(_.info).Get("*", info)
-    return res
-predefined_sources.register(get_kegg_pathway,name="pathway", category="kegg")
-
+                id = elem[:pos]
+                info = elem[(pos+1):].split('; ')
+                name_info.append((id, info[0],info[1:]))
+    dtype = '[idinfo:*]<(id=string, full_name=string, info=[info_elems:~]:string)'
+    return Rep(name_info,dtype=dtype)/("id", "full_name","info")
+predefined_sources.register(get_kegg_btit,name="id_info", category="kegg")
 
 def extract_info(vals):
     names, info = vals
