@@ -22,7 +22,6 @@ _delay_import_(globals(),"repops_funcs")
 _delay_import_(globals(),"ops")
 
 
-
 class NewDim(object):
     def __init__(self,name=None):
         self.name = name
@@ -138,9 +137,6 @@ class Representor(Node):
             res += tableprint.indent(srows,hasHeader=True) 
         return res 
 
-
-
-
     def _getInfo(self):
         class Info(object):
             def __repr__(self):
@@ -259,6 +255,10 @@ class Representor(Node):
                     tuple([slice.name for slice in self._slices]))
     Type=property(fget=getType)
 
+
+    def Redim(self, *args, **kwds):
+        return repops_dim.Redim(self, *args, **kwds)
+
     def getSlices(self):
         return list(self._slices)
     Slices=property(fget=getSlices)
@@ -352,7 +352,7 @@ class Representor(Node):
     def __call__(self, **args):
         res = self._getResultSlices(args)
 
-        if(len(self._slices) == 1):
+        if(len(res) == 1):
             return res[0].data
         else:
             return tuple([slice.data for slice in res])
@@ -683,9 +683,15 @@ class Representor(Node):
     def Pos(self, dim=None):
         return repops_funcs.Pos(self, dim)
     
+    def CumSum(self, dim=None):
+        return repops_funcs.CumSum(self, dim)
+   
     def Argsort(self, dim=None, descend=False):
         return repops_funcs.Argsort(self, dim, descend=descend)
     
+    def Rank(self, dim=None, descend=False):
+        return repops_funcs.Rank(self, dim, descend=descend)
+   
     def Argunique(self, dim=None):
         return repops_funcs.Argunique(self, dim)
 
@@ -760,8 +766,8 @@ class Representor(Node):
         """Combines slices into a tuple type"""
         return repops_slice.IndexDict(self)
 
-    def TakeFrom(self, other, allow_missing=False):
-        return repops_multi.Take(other, self, allow_missing)
+    def TakeFrom(self, other, allow_missing=False,keep_missing=False):
+        return repops_multi.Take(other, self, allow_missing,keep_missing)
 
     def ToPython(self):
         """Converts data into python data structure"""
