@@ -3,10 +3,50 @@ import hashlib
 from itertools import izip
 import operator
 import collections
-from IPython.Debugger import Tracer; debug_here = Tracer()
+#from IPython.Debugger import Tracer; debug_here = Tracer()
 import cPickle, zlib
 import math
 import os.path
+import gc, sys
+import itertools
+import csv
+
+
+#objs: objs to find names for
+#req_objs: objs which should be in same frame
+def find_names(objs, req_objs=[],skip_frames=True):
+    #after: http://pythonic.pocoo.org/2009/5/30/finding-objects-names
+    result = []
+    referrers = dict()
+    refsetobjs = []
+
+    frame = sys._getframe()
+    #refs should be in locals dict
+    frameobj = []
+    for fcounter,frame in enumerate(iter(lambda: frame.f_back, None)):
+        fid = id(frame.f_locals)
+        if not skip_frames or fcounter >= 1:
+            referrers[fid] = frame.f_locals
+            frameobj.append(fid)
+    refsetobjs.append(set(frameobj))
+
+    for obj in itertools.chain(objs,req_objs):
+        refsetobj = [id(referrer) for referrer in gc.get_referrers(obj) if isinstance(referrer, dict)]
+        refsetobjs.append(set(refsetobj))
+    refs_with_all_objs = reduce(operator.__and__,refsetobjs)
+
+    for refid in frameobj:
+        if not refid in refs_with_all_objs:
+            continue
+        referrer = referrers[refid]
+        result = []
+        for obj in objs:
+            for k, v in referrer.iteritems():
+                if v is obj and not k.startswith('_') and not k[:2].isupper():
+                    result.append(k)
+        if(len(result) == len(objs)):
+            return result
+    return []
 
 _delay_import_(globals(),"missing","Missing")
 def save_rep(r, filename):
@@ -15,11 +55,24 @@ def save_rep(r, filename):
     s = zlib.compress(s)
     f.write(s)
 
+
+def save_csv(r, filename):
+    f = open(filename,'wb')
+    r= r.Array(tolevel=1)
+    data = r.Tuple().ToPython();
+    if filename.endswith('tsv'):
+        w = csv.writer(f,delimiter='\t');
+    else:
+        w = csv.writer(f);
+    w.writerow(r.Names);
+    w.writerows(data);
+
 def load_rep(filename):
     f = open(filename, 'rb')
     s = zlib.decompress(f.read())
     return cPickle.loads(s)
 
+
 def valid_name(name):
     name = name.lower()
     newname = []
