@@ -85,7 +85,11 @@ def Rep(data=None, dtype=None, unpack=True, name=None):
             dtype = rtypes.TypeAny(True)
 
     if(name is None):
-        name = "data"
+        res = util.find_names([data])
+        if res:
+            name = res[0]
+        else:            
+            name = "data"
 
     data_slice = ops.DataOp(data,name=name,rtype=dtype)
     
@@ -305,7 +309,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
        
     def visitTakeOp(self, node, slices):
         ndata = nested_array.co_mapseq(speeddictindex,[slice.data for slice in slices],res_type=node.type, 
-                                        dtype=node.type.toNumpy(), allow_missing=node.allow_missing)
+                                        dtype=node.type.toNumpy(), allow_missing=node.allow_missing, keep_missing = node.keep_missing)
         return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
 
     def visitPackDictOp(self,node, slices):
@@ -630,7 +634,8 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
         res = []
         dtype = typeo.toNumpy()
         for lelem, relem in zip(data1,data2):
-            res.append(numpy.concatenate([numpy.cast[dtype](list(lelem)),numpy.cast[dtype](list(relem))],axis=0))
+            mshape = max(1,min(len(getattr(lelem,'shape',(0,))), len(getattr(relem,'shape',(0,)))))
+            res.append(numpy.concatenate([cutils.darray(list(lelem),dtype,mshape),cutils.darray(list(relem),dtype,mshape)],axis=0))
         return cutils.darray(res,object)
 
     def simple_cmpGeneral(self, data, type1, type2, typeo, op):
@@ -693,6 +698,15 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
                 res.append(elem)
         return cutils.darray(res, type_out.toNumpy())
     
+    def ismissingIsMissing(self, data, type_in, type_out, op, def_value=NOVAL):
+        res = []
+        for elem in data:
+            if elem is Missing:
+                res.append(True)
+            else:
+                res.append(False)
+        return cutils.darray(res, type_out.toNumpy())
+   
     def corrCorr(self, data, type_in, type_out, op):
         intype = type_in.toNumpy()
         res = []
@@ -720,7 +734,29 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
             if(descend):
                 res = res[:,::-1,...]
         return res
-   
+    
+    def sortableRank(self, data, type_in, type_out, op, packdepth, descend=False):
+        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
+        if(len(data.shape) < 2):
+            if(descend):
+                res = cutils.darray([numpy.argsort(numpy.flipud(numpy.argsort(row,axis=0)),axis=0) for row in data],object)
+            else:
+                res = cutils.darray([numpy.argsort(numpy.argsort(row,axis=0),axis=0) for row in data],object)
+        else:
+            res = numpy.argsort(data,axis=1)
+            if(descend):
+                res = res[:,::-1,...]
+            res = numpy.argsort(res,axis=1)                
+        return res
+    
+    def fixdimCumSum(self, data, type_in, type_out, op, packdepth):
+        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
+        if(len(data.shape) < 2):
+            res = cutils.darray([numpy.cumsum(row,axis=0) for row in data],object)
+        else:
+            res = numpy.cumsum(data,axis=1)
+        return res
+ 
     def any_nodepPos(self, data, type_in, type_out, op, packdepth):
         dtype = type_out.toNumpy()
         if(len(data.shape) == 1):
@@ -919,8 +955,10 @@ def speedarrayify(seqs,dtype):
     return nseq
 
 
-def speeddictindex(seqs,dtype, allow_missing=False):
-    if allow_missing:
+def speeddictindex(seqs,dtype, allow_missing=False, keep_missing=False):
+    if keep_missing:
+        return cutils.darray([d.get(k,k) for d, k in zip(*seqs)],dtype)
+    elif allow_missing:
         return cutils.darray([d.get(k,Missing) for d, k in zip(*seqs)],dtype)
     else:
         return cutils.darray([d[k] for d, k in zip(*seqs)],dtype)
@@ -930,7 +968,7 @@ def speedfilter(seqs,has_missing, ctype, stype):
     data,constraint = seqs
     if data is Missing:
         return data
-    if len(data) == 0:
+    if len(data) == 0: #FIXME
         return data
     if(has_missing):
         if(isinstance(ctype,rtypes.TypeArray)):
